#ifndef __ANL_PKT
#define __ANL_PKT
#include <time.h>
#include "hdr_type.h"
#include "capt.h"
class AnlPkt {
  protected:
    AnlPkt * next;
    // Vlastni metody
    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut)=0;

  public:
    // Def. konstruktor
    AnlPkt() {
      next = NULL; // Pro jistotu
    }

    // Destruktor
    virtual ~AnlPkt() {
      chk_mem("AnlPkt->del 0x%hX\r\n", get_types());
      if (next) {
	delete next;
      }
    }

    //Tridni metody
    static AnlPkt * parse_pkt(const uchar * pdata, pkthdr *ph,
						   fhdr * hdr = NULL);
    // Virtualni metody
    virtual void show_prot() { }
    virtual void header(ushort flags, char recursive=TRUE) {  }
    virtual void show(ushort flags)=0;
    virtual ushort get_types() = 0;


};

class PktMain : public AnlPkt {
  protected:
    static uint32 count;
    uint32 ltype; /* Typ spoj. vrstvy => preskoceni */
    uint32 usec;
    tm t;

    //Instancni metody
    void init_hdr(fhdr * hdr) {
      if (hdr) {
	set_ltype(hdr->linktype);
      } else {
	def_ltype();
      }
      next = NULL; // Pro jistotu
    }
    // Vlastni metody
    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut);

  public:

    PktMain(const uchar * pktdata, pkthdr *ph, fhdr * hdr=NULL) {
      init_hdr(hdr);
      if (ph) { // Not NULL
	 PktMain::count++;
	 usec = ph->tstamp.usec;
	 t = * gmtime((time_t *)&(ph->tstamp.time));
	 next=parse_pkt(pktdata, ph->caplen, ph->caplen !=  ph->len);
      }
    }

    // Instancni metody

    void def_ltype() {
      set_ltype(DLT_EN10MB);
    }

    void set_ltype(uint32 lt) {
      ltype = lt;
    }

    // Virtualni metody
    virtual void header(ushort flags, char recursive=TRUE) {
      if ( flags & T_INFO ) {
	g_printf("# %lu", count);
	if ( S_TIME | S_DATE | S_USEC) {
	  g_puts(" (");
	  if ( flags & S_DATE) {
	    g_printf("%02u.%02u.%04u ", t.tm_mday, t.tm_mon+1, t.tm_year+1900);
	  }
	  if ( flags & (S_TIME|S_DATE)) { // samostatne datum asi ne?
	    g_printf("%02u:%02u:%02u", t.tm_hour, t.tm_min, t.tm_sec);
	  }
	  if ( flags & S_USEC) {
	    g_printf(".%06lu", usec);
	  }
	  g_puts(" GMT)");
	}
      }
      if (recursive && next) {
	g_puts(": ");
	next->header(flags);
      }
    }

    virtual void show(ushort flags) {
      if (flags & T_INFO) {
	header(flags, FALSE);
	g_putc('\n');
      }
      if (next) {
	next->show(flags);
#ifdef MEMDBG
      } else {
	g_puts("Nedefinovana hodnota next v AnalPkt ci potomkovi\n");
#endif
      }
    }

    virtual ushort get_types() {
      return ((next)?(next->get_types()):0) | T_INFO;
    }

};

inline AnlPkt * AnlPkt::parse_pkt(const uchar * pdata,
					 pkthdr *ph, fhdr * hdr) {
   return new PktMain(pdata, ph, hdr);
}

class AdrPkt : public AnlPkt {
  public:
    virtual void show_a(uchar kind) = 0;
};

class PortPkt : public AnlPkt {
  protected:
    virtual char is_known(ushort port1, ushort port2, ushort type) {
      return (getport(port1, type) || getport(port2, type));
    }

    virtual void show_port(ushort port, ushort type) {
      if (! port) {
	g_puts(" Zadny");
      } else {
	const char * pnam = getport(port, type);
	if ( pnam ){
	  g_printf(" %s", pnam);
	} else {
	  g_printf(" %hu", port);
	}
      }
    }
    virtual uchar inter_p(ushort port, ushort type, uchar dir) {
      return interresting(port, type, dir);
    }
  public:
    virtual void show_p(uchar kind) = 0;
} ;

#endif /* ANL_PKT */
