#ifndef __ARP_PKT
#define __ARP_PKT
#  include  "anl_pkt.h"
#  include  "gen_pkt.h"

class ARP_Pkt : public AdrPkt {

protected:
    ARP_hdr * hdr;
    enum {IP=0x800, RARP_MIN=3, RARP_MAX=7, FROM_C=1, TO_C=9}; 
    enum {ARP_rqs =1, ARP_rsp, RARP_rqs, RARP_rsp, DRARP_rqs, DRARP_rsp,
          DRARP_err, InARP_rqs, InARP_rsp};
    // Vlastni metody

    void swap_hdr() {
      if (!hdr) {
        return;
      }
      hdr->h_type = SWAP16(hdr->h_type);
      hdr->l_type = SWAP16(hdr->l_type);
      hdr->opcode = SWAP16(hdr->opcode);
    }

    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut) {
      if ( psize >= sizeof(ARP_hdr) ) {
        hdr = (ARP_hdr*) new char[psize];
        memcpy(hdr, pdata, psize);
	     swap_hdr();
      }
      return NULL;
    }

   virtual void show_prot();
public:

    static void show_addr(uchar * adr, uchar size, ushort type) {
      show_MAC(adr, size, type);
    }
    // Konstruktory
    ARP_Pkt(const uchar * pktdata, const uint32 psize, char cut) {
      hdr = NULL;
      next=parse_pkt(pktdata, psize, cut);
    }

    // Destruktor
    virtual ~ARP_Pkt() {
   	chk_mem("ARP_Pkt->del 0x%hX\r\n", get_types());
	   if (hdr) {
	     delete hdr;
	   }
    }

    // Instancni metoda
    void show_a(uchar kind, uchar swap) { // swap - TRUE/FALSE - prehozeno?
      if (!hdr) {
        return;
      }
      if ( kind == SRC ) {
        if ( (hdr->opcode < RARP_MIN || hdr->opcode > RARP_MAX) != swap) {
  	       show_addr(hdr->adr+hdr->h_len, hdr->l_len, T_IP);
        } else {
  	       show_addr(hdr->adr, hdr->h_len, T_FRAME); 
        } 
      } else if ( kind == DST ) {
        if ((hdr->opcode < RARP_MIN || hdr->opcode > RARP_MAX) != swap) {
  	       show_addr(hdr->adr+(2*hdr->h_len)+hdr->l_len, hdr->l_len, T_IP);
        } else {
  	       show_addr(hdr->adr+hdr->h_len+hdr->l_len, hdr->h_len, T_FRAME);
        } 
      }
    }
    
    // Virtualni metody
    virtual void show_a(uchar kind) {
       show_a(kind, FALSE);
    }

    virtual void header(ushort flags, char recursive) { 
      if ( hdr && (flags & T_ARP)) {
        show_prot();
      }
    }

    virtual void show(ushort flags) {
      if ( hdr && (flags & T_ARP)) {
        header(flags, FALSE);
        g_puts(": ");
        if ( hdr->opcode == DRARP_err) {
           g_puts("HW adresu nelze urcit");
        } else if ( hdr->opcode == ARP_rsp || hdr->opcode == RARP_rsp || 
                    hdr->opcode == InARP_rsp || hdr->opcode == DRARP_rsp ) {
          g_puts("adresa ");
	       show_a(SRC);
	       g_puts(" (pro ");
	       show_a(DST);
          g_puts(") je ");
	       show_a(SRC, TRUE); // fyz.<->log. adr
        } else { // Pozadavek
          g_puts("kdo je ");
	       show_a(DST);
    	    g_puts(" (pro ");
  	       show_a(SRC);
          g_puts(" - ");
  	       show_a(SRC, TRUE); // fyz.<->log. adr
          g_puts(")?");
        }  
        g_putc('\n');
      }
    }

    virtual ushort get_types() {
      return ((next)?(next->get_types()):0) | T_ARP;
    }
};


#endif /* ARP_PKT */
