#ifndef __CAPT_H
#define __CAPT_H

#if defined (__TURBOC__) && \
    !( defined(__HUGE__) || defined(__COMPACT__) || defined(__LARGE__))
#    error Huge, Compact or Lage model required
#endif
#if (defined(__TURBOC__) || defined(__DJGPP__)) && ! defined(STDOUT)
#  include <conio.h>
#  define g_puts cputs
#  define g_putc putch1
#  define g_printf cprintf
#else
#  define g_puts printf
#  define g_putc putchar
#  define g_printf printf
#endif

#ifdef MEMDBG
# define chk_mem(x,y) g_printf(x,y)
#else
# define chk_mem(x,y)
#endif

#include <stdio.h> /* Kvuli `FILE *' */
#include <string.h>

/* Konfigurace z libpcap 2.4 - pro zajisteni kompatibility */
#define MAGIC 0xa1b2c3d4l
#define DEF_SIGFIGS 0

#define MAGIC2 0xa1b2cd34 /* Modifikovana knihovna                  */
#define MODLIB_SKIP 8     /* Preskoceni hlavicek v mod. knihovne    */

#define MAJOR 2
#define MINOR 4
#define MIN_MINOR 3       /* Pred touto verzi byl zmenen format dat */

#define DLT_EN10MB 1      /* Kod hlavicky pro ethernet */
#define HDR_EN10MB 14     /* Delka hlavicky ethernet   */

#define DEF_SNAP 0x90     /* Std. delka snapshotu  */

#ifndef TRUE
#  define TRUE 1
#  define FALSE 0
#endif

typedef unsigned long uint32;
typedef signed long int32;
typedef unsigned short ushort;
typedef unsigned char uchar;

typedef struct {
   uint32 time;       /* Syst. cas v sekundach                          */
   uint32 usec;       /* Mikrosekundy - nelze v DOSu ziskat ?,
			 tj. jen milisec?                               */
} time_val;


typedef struct {
  uint32 magic;        /* Magicke cislo (z MAGIC)                        */
  ushort major, minor; /* Hlavni a vedlejsi verze (MAJOR, MINOR)         */
  int32 tzone;         /* Casove pasmo - rozdil sekund vuci GMT          */
  uint32 sigfigs;      /* nastaveno na 0                                 */
  uint32 snaplen;      /* max. delka ulozeneho ramce (def. DEF_SNAP)     */
  uint32 linktype;     /* typ ramce LLC - zde vzdy Ethernet(10Mb), tj. 1 */
} fhdr;

typedef struct {
  time_val tstamp;     /* Syst. cas, vcetne ms, mikrosekundy v dosu nelze*/
  uint32 caplen;       /* Ulozenych byte ramce                           */
  uint32 len;          /* Celkem byte ramce                              */
} pkthdr;

typedef struct {
  pkthdr * h;          /* Hlavicka paketu */
  uchar * data;        /* Data     "      */
} pktrec;

enum { OK=0, FILEERR, MEMERR, CARDERR /*, ... */};
/* Navratove kody - errorlevel */

#define	SWAP32(y) \
((((y)&0xff)<<24) | (((y)&0xff00)<<8) | (((y)&0xff0000l)>>8) | (((y)>>24)&0xff))
#define	SWAP16(y) ( (((y)&0xff)<<8) | ((ushort)((y)&0xff00)>>8) )

#ifdef __cplusplus
extern "C" {
#endif
FILE * pkt_wr_open (char * fname, fhdr * hdr, uint32 snaplen);
FILE * pkt_rd_open (char * fname, fhdr ** hdr);
pktrec * pkt_read(FILE* fil);
void pkt_write(FILE* fil, pktrec * pkt);

void hexshow(uchar * data, uint32 size, char full);
void hexdump(uchar * data, uint32 size, char sep, ushort el_per_line);
void show_MAC(uchar * adr, uchar size, ushort type);
void swap_phdr(pkthdr *);
void swap_fhdr(fhdr *);

void freepkt(pktrec *);
int putch1(int);
const char * getport(ushort, ushort);
uchar interresting(ushort portNo, ushort proto, uchar kind);
#ifdef __cplusplus
}
#endif

enum { T_FRAME=1, T_IP = 2, T_TCP = 4, T_UDP = 0x8,
       T_ICMP=0x10, T_SERV = 0x20, T_DATA=0x40, T_OTHERS=0x80,
       T_INFO=0x100,  S_DATE=0x200, S_USEC=0x400, S_TIME=0x800,
       S_INTER=0x1000, T_ARP=0x2000, T_ALL = 0xffff
};

enum { NEVER=0, SOMETIMES=1, ALWAYS=3 };
enum {DST=1, SRC=2}; /* Zdrojovy/a, cilovy/a port/adresa */

#endif /* __CAPT_H */
