#include "pkt_all.h"
#include "arp_pkt.h"
EthFrame::EthFrame(const uchar * pktdata, const uint32 psize, char cut, ushort type) {
  for (int i=0; i < ADDRSIZ; i++) {
    src[i] = dest[i] = '\0';
  }
  prot = type;
  next=parse_prot(pktdata, psize, cut);
}

AnlPkt * EthFrame::parse_prot(const uchar * pdata, uint32 psize, char cut) {
  switch (prot) {
     case IP:     // IP protocol
             return new IP_Pkt(pdata, psize, cut);
     case ARP:
     case RARP:     // (R)ARP protocol
             return new ARP_Pkt(pdata, psize, cut);
     default:
             return new Other_Pkt(pdata, psize, cut);
  }
}

AnlPkt * EthFrame::parse_pkt(const uchar * pdata, uint32 psize, char cut) {
  char skip = HDR_EN10MB;
  if (psize >= DLT_EN10MB) {
    memcpy(dest, pdata,ADDRSIZ);
    memcpy(src, pdata+ADDRSIZ, ADDRSIZ);
    prot = (pdata[PROTPOS] << 8) | pdata[PROTPOS+1];
    if ( prot < MIN_PROT ) { // Ethernet 802.3
      prot = (pdata[PROTPOS+2] << 8) | pdata[PROTPOS+3];
      skip += 2;
    }
    pdata += skip;
    psize -= skip;
  }
  return parse_prot(pdata, psize, cut);
}

void EthFrame::show_prot() {
   g_puts(" (");
   switch (prot)  {
     case IP: 		g_puts("IP");
			break;
     case ARP:       	g_puts("ARP");
			break;
     case RARP:		g_puts("RARP");
			break;
     case IPX0:
     case IPX1:
     case IPX2:
     case IPX3:         g_puts("IPX");
			break;
     case SPT:		g_puts("SPT");
			break;
     case SNMP:		g_puts("SNMP");
			break;
     case IPv6:		g_puts("IPv6");
			break;
     case NetBIOS:	g_puts("NetBIOS");
			break;
     case DECNet:	g_puts("DECNet");
			break;

     default:
		g_printf("0x%hX", prot);
   }
   g_puts(") ");
}

void ARP_Pkt::show_prot() {
  char * nazvy[] = { "ARP - poz.", "ARP - odp.", 
                     "RARP - poz.", "RARP - odp.",     
                     "DRARP - poz.", "DRARP - odp.", "DRARP - chyba", 
                     //dynamicky RARP ^
                     "InARP - poz.", "InARP - odp."
  };
  if (hdr->opcode < FROM_C || hdr->opcode > TO_C) {
     g_printf("ARP/RARP(0x%04hX)", hdr->opcode);
  } else {
     g_puts(nazvy[hdr->opcode-1]);
  }   
}
