#ifndef __ETH_FRAME
#define __ETH_FRAME
#  include  "anl_pkt.h"

class EthFrame : public AdrPkt {

protected:
    enum {A_DST=0, A_SRC=1, ADDRSIZ=6, PROTPOS=12, MIN_PROT=0x600};
    enum {IP=0x800, ARP=0x806, RARP=0x8035, IPX0=0x8037, IPX1=0x1237,
	  IPX2=0x1238, IPX3=0x8137, SNMP=0x814C, IPv6=0x86DD, SPT=0x4242,
	  DECNet=0xAAAA, NetBIOS=0xF0F0};
    uchar dest[ADDRSIZ], src[ADDRSIZ]; // adresy
    ushort prot;

    // Vlastni metody
    virtual AnlPkt * parse_prot(const uchar * pdata, uint32 psize, char cut);

    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut);

    virtual void show_addr(uchar * addr, int size=ADDRSIZ) {
      hexdump(addr, size, ':', size+1);
    }

public:
    // Konstruktory
    EthFrame(const uchar * pktdata, const uint32 psize, char cut, ushort type);

    EthFrame(const uchar * pktdata, const uint32 psize, char cut) {
      next=parse_pkt(pktdata, psize, cut);
    }

    // Virtualni metody
    virtual void show_prot();

    virtual void show_a(uchar kind) {
      if ( kind == SRC ) {
	show_addr(dest, ADDRSIZ);
      } else if ( kind == DST ) {
	show_addr(src, ADDRSIZ);
      }
    }
    
    virtual void header(ushort flags, char recursive=TRUE) {
      if ( flags & T_FRAME) {
	show_addr(src);
	g_puts(" -> ");
	show_addr(dest);
	show_prot();
      }
      if (recursive && next) {
        next->header(flags);
      }
    }

    virtual void show(ushort flags) {
      if ( flags & T_FRAME) {
	g_puts("Ramec: ");
        header(flags, FALSE);
        g_putc('\n');
      }
      if (next) {
	next->show(flags);
      }
    }

    virtual ushort get_types() {
      return ((next)?(next->get_types()):0) | T_FRAME;
    }


};

#endif /* ETH_FRAME */
