#include "pkt_all.h"
char * IP_Pkt::Speeds [] = { "bezny provoz", "s prioritou", "co nejdriv", 
                             "bleskove", "bleskove++", "kriticka/ECP", 
                             "sprava propoj. site", "sprava site"        };

AnlPkt * IP_Pkt::parse_pkt(const uchar * pdata, uint32 psize, char cut) {
  IP_hdr *h = (IP_hdr *) pdata;
  if ( h->ver != 4 ) { // jakakoliv jina verze - prip. anal IPv6
    /*
       Zde by slo dodelat zprac IPv6, pri vypisu a zjisteni teto verze
       hlavicku (nutno mit spravnou delku) pretypovat a pracovat s ni
    */
    hdr = new IP_hdr();
    hdr->ver = h->ver;
  } else {
    int skip = h->ihl << 2 ; // Vel. hlavicky (delena 4)
    if (skip <= psize) {
      uchar * hdata = new uchar [skip];
      hdr = (IP_hdr*) memcpy(hdata, pdata, skip);
      swap_hdr();
      pdata += skip;
      psize -= skip;
    }
    // TBD
  }
  switch (hdr->protocol) {
     case 1:   return new ICMP_Pkt(pdata, psize, cut);	   // ICMP
     case 6:    return new TCP_Pkt(pdata, psize, cut);	   // TCP
     case 17:   return new UDP_Pkt(pdata, psize, cut);	   // UDP

     // Bez zaruky
     case 4:
     case 94:  return new IP_Pkt(pdata, psize,cut);   // IP in IP
     case 97:  return new EthFrame(pdata, psize,cut); // Ethernet zabaleny v IP
  }
  return new Other_Pkt(pdata, psize, cut);
}

void IP_Pkt::header(ushort flags, char recursive) {
  if ( (flags & T_IP) || ! ( flags & T_FRAME) ) {
    if (hdr) {
      switch (hdr->ver) {
	case 6: // IPv6
		g_puts("IPv6 - neanalyzovano");
		return; // next_header uz ani nemuze nastat
	case 4: // IPv4 - OK
		break;
	default: // Jina verze IP - zahada
		g_printf("IP(%u) ???", hdr->ver );
		return; // next_header uz ani nemuze nastat
      }
      show_addr(hdr->src);
      g_puts(" -> ");
      show_addr(hdr->dest);
      show_prot();
    } else {
      g_puts("IP hlavicku nelze zobrazit (asi byla oriznuta)\n");
    }
    if (recursive && next) {
      next->header(flags);
    }
  }
}

void IP_Pkt::show(ushort flags) {
  if ( hdr && ((flags & T_IP) || ! ( flags & T_FRAME)) ) {
    g_puts("IP: ");
    header(flags, FALSE);
    if (!hdr || hdr->ver != 4 ){ 
      // Jina verze protokolu nez 4, nebo nezobrazovat hlavicky IP -> konec
      g_putc('\n');
      return;
    }
    
    if(!(flags & T_IP) && next) {
      g_putc('\n');
      next->show(flags);
      return;
    }
      
    g_printf(" Delka: %hu (hl. %hu)", hdr->t_len, hdr->ihl << 2 );
    g_printf(" ID: %hu", hdr->ident); // Id
    g_putc('\n');
    g_puts("    ");
    if ( hdr->flg_off.fo.flags & 2 ) {
      g_printf("Nefragmentovat! ");
    } else {
      g_printf("Offset frag.: 0x%04hX " , hdr->flg_off.fo.offset); // Offset
    }

    if ( hdr->flg_off.fo.flags & 1 ) {
      g_printf("Posledni fragment ");
    }
    g_printf(" CHK: 0x%04hX", hdr->chksum);
    if ( hdr->tos & 0x1C) {
      g_puts(" Preferovat: ");
      (hdr->tos & 0x10)  && g_puts("rychlost ");
      (hdr->tos & 0x08)  && g_puts("propustnost ");
      (hdr->tos & 0x04)  && g_puts("bezpecnost ");
    }
    g_putc('\n');
    g_puts("    Priorita: ");
    int prio=(hdr->tos >> 5) & 7;
    g_printf("%s (%hu)", Speeds[prio],prio);
    g_printf("  Zivotnost: %u", hdr->t_to_live); //  TTL
    g_putc('\n');
  }
  if (next) {
    next->show(flags);
  }
}

void IP_Pkt::show_addr(uchar * adr, int size) {
  if (!hdr) { return; }
  g_printf("%i",adr[0]);
  for (int i=1; i < size; i++) {
    g_printf(".%i",adr[i]);
  }
}

void IP_Pkt::show_prot() {
  if (!hdr) { return; }
  g_puts(" (");
  switch (hdr->protocol) {
     case 1: 	g_puts("ICMP");       // Internet control message (ping, ...)
		break;
     case 2: 	g_puts("IGMP");       // Internet group management
		break;
     case 3: 	g_puts("GGP");        // Gateway 2 Gateway
		break;
     case 4:
     case 94:	g_puts("IP-in-IP");   // IP in IP
		break;
     case 5:	g_puts("Stream");     // Stream - RFC1190, ...
		break;
     case 6:	g_puts("TCP");        // TCP
		break;
     case 11:	g_puts("NVP-II");     // Network voice protocol
		break;
     case 17:	g_puts("UDP");        // UDP
		break;
     case 97: 	g_puts("Ethernet");   // Ethernet zabaleny v IP
		break;
     case 103:	g_puts("PIM");   // Ethernet zabaleny v IP
		break;
     default:	g_printf("%hd", hdr->protocol);
  }
  g_putc(')');
}
