#include "capt.h"
#include <stdlib.h>
#ifdef __TURBOC__
#  include <alloc.h>
  extern long timezone;
#else
  long timezone=0; /* Stejne to je asi default. format, s TZ se dal neprac?*/
#ifdef __DJGPP__
#   define coreleft stackavail
#else
#define coreleft() 0
#endif
#endif

fhdr def_fhdr = { MAGIC, MAJOR, MINOR, 0, DEF_SIGFIGS, 0, DLT_EN10MB};
char swapped = FALSE; /* Prehozena data, nutna konverze endianu - 1 */
char modlib = FALSE;  /* Modifikovana verze libpcap                 */
/*ushort hdr_skip = HDR_EN10MB;*/

void swap_phdr(pkthdr *hdr) {
  hdr->tstamp.time = SWAP32(hdr->tstamp.time);
  hdr->tstamp.usec = SWAP32(hdr->tstamp.usec);
  hdr->caplen  = SWAP32(hdr->caplen);
  hdr->len     = SWAP32(hdr->len);
}

void swap_fhdr(fhdr *hdr) {
  hdr->magic = SWAP32(hdr->magic);
  hdr->major = SWAP16(hdr->major);
  hdr->minor = SWAP16(hdr->minor);
  hdr->tzone = SWAP32(hdr->tzone);
  hdr->sigfigs = SWAP32(hdr->sigfigs);
  hdr->snaplen = SWAP32(hdr->snaplen);
  hdr->linktype = SWAP32(hdr->linktype);
}

FILE * pkt_rd_open (char * fname, fhdr ** hdr) {
  FILE * fil;
  if (*hdr == NULL) {
    *hdr = (fhdr *) malloc (sizeof (fhdr));
  }
  fil = fopen(fname, "rb");
  if ( fil ) {
    fread(*hdr, sizeof(fhdr), 1, fil);
    if ( (*hdr)->magic != MAGIC && (*hdr)->magic != MAGIC2) {
      swap_fhdr(*hdr);
      if ( (*hdr)->magic != MAGIC && (*hdr)->magic != MAGIC2) {
	fprintf(stderr, "Chybny format dat. souboru %s (MAGIC)\n", fname);
	exit(FILEERR);
      } else {
	swapped = TRUE;
      }
    }
  } else {
    fprintf(stderr, "Nelze otevrit soubor %s pro cteni\n", fname);
    exit(FILEERR);
  }
  if ((*hdr)->magic == MAGIC2) { /* Modifikovana verze knihovny */
    modlib = TRUE;
  }
  if ( (*hdr)->major < MAJOR /*|| (*hdr)->minor < MIN_MINOR */) {
    fprintf(stderr, "Soubor %s byl vytvoren ve verzi libpcap %i.%i, "
		    "ocekavana alespon %i.%i\n",
	 fname, (*hdr)->major, (*hdr)->minor, MAJOR, MIN_MINOR
    );
    exit(FILEERR);
  }
  return fil;
}

FILE * pkt_wr_open (char * fname, fhdr * hdr, uint32 snaplen) {
  FILE * fil;
  if (! hdr->snaplen) { /* Nenastaveno - default header, prevzate nemod. */
    hdr->tzone = timezone;
    if ( snaplen > DEF_SNAP) {
      hdr->snaplen = snaplen;
    } else {
      hdr->snaplen = DEF_SNAP;
    }
  }
  if ( hdr->magic != MAGIC ) { /* Modifikovana knihovna */
    hdr->magic = MAGIC;
    hdr->sigfigs = DEF_SIGFIGS;
  }

  fil = fopen(fname, "wb");
  if ( fil ) {
    fwrite(hdr, sizeof(*hdr), 1, fil);
  } else {
    fprintf(stderr, "Nelze otevrit soubor %s pro zapis\n", fname);
    exit(FILEERR);
  }
  return fil;
}

void pkt_write(FILE* fil, pktrec * pkt) {
   fwrite(pkt->h, sizeof(pkthdr), 1, fil);
   fwrite(pkt->data, pkt->h->caplen, 1, fil);
}

pktrec * pkt_read(FILE* fil) {
   int data_siz;
   pktrec * pkt = (pktrec*) malloc(sizeof(pktrec));
   pkthdr * pkth = (pkthdr*) malloc(sizeof(pkthdr));
   if (pkt && pkth ) {
     pkt->h = pkth;
     if (!fread(pkt->h, sizeof(pkthdr), 1, fil)) { /* EOF */
       free(pkth);
       free(pkt);
       return NULL;
     }
     if (modlib) {
       fseek(fil, MODLIB_SKIP, SEEK_CUR);
     }
   } else {
      fprintf(stderr, "Nedostatek pameti pro ulozeni paketu\n");
      exit(MEMERR);
   }
   if ( swapped ) {
     swap_phdr(pkt->h);
   }
   if ( pkt->h->len < pkt->h->caplen ) { /* v nekterych verzich libpcap */
     uint32 len = pkt->h->caplen;
     pkt->h->caplen = pkt->h->len;
     pkt->h->len = len;
   }
   pkt->data = (uchar *) malloc(pkt->h->caplen);
   if (pkt->data || !(pkt->h->caplen)) {
     if ((fread(pkt->data, 1, pkt->h->caplen, fil)) < pkt->h->caplen ) {
       /* EOF  pred koncem ulozeneho - napr. po Ctrl-C*/
       pkt->h->caplen = 0;
       fprintf(stderr, "Datova cast paketu konci pred koncem souboru\n");
     }
   } else {
      fprintf(stderr,"Nedostatek pameti (%lu) pro paket o delce %lu\n",
		      coreleft(), pkt->h->caplen);
      exit(MEMERR);
   }
   return pkt;
}

void freepkt(pktrec * pkt) {
  if (pkt) {
    if ( pkt->data ) {
      free(pkt->data);
    }
    if (pkt->h) {
      free(pkt->h);
    }
    free(pkt);
  }
}
