#ifndef __TCP_PKT
#define __TCP_PKT
#  include  "anl_pkt.h"
#  include  "gen_pkt.h"

class TCP_Pkt : public PortPkt {

protected:
    TCP_hdr * hdr;
    uint32 siz;
    // C. portu, pocet flagu, posledni v cyklu
    enum {ADDRSIZ=2, FLG_CNT=6, FLG_LAST=4};
    enum {F_FIN=1, F_SYN=2, F_RST=4, F_PSH=8, F_ACK=0x10, F_URG=0x20};
    static const char * Flg_n [];
    // Vlastni metody

    void swap_hdr() {
      if (!hdr) {
        return;
      }
      hdr->d_port = SWAP16(hdr->d_port);
      hdr->s_port = SWAP16(hdr->s_port);
      hdr->flg_off.rep = SWAP16(hdr->flg_off.rep);
      hdr->window = SWAP16(hdr->window);
      hdr->chksum = SWAP16(hdr->chksum);
      hdr->urgent_ptr = SWAP16(hdr->urgent_ptr);

      hdr->seqNo  = SWAP32(hdr->seqNo);
      hdr->ackNo  = SWAP32(hdr->ackNo);
    }

    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut) {
      siz = psize;
      TCP_hdr *h = (TCP_hdr *) pdata;
      ushort hsiz = (h->flg_off.rep & 0xf0) >> 2; // Rucni BSWAP
      if (hsiz < sizeof(TCP_hdr)) {
	hsiz = psize+1;
	hdr = NULL;
      }
      if ( psize >= hsiz ) {
	hdr = (TCP_hdr *) new char[hsiz];
	memcpy(hdr, pdata, hsiz);
	pdata += hsiz;
	psize -= hsiz;
	swap_hdr();
      }
      if ( psize ) {
	return new Pkt_Data(pdata, psize, cut);
      } else {
	return NULL; // Prazdny paket
      }
    }

public:
    // Konstruktory
    TCP_Pkt(const uchar * pktdata, const uint32 psize, char cut) {
      hdr = NULL;
      next=parse_pkt(pktdata, psize, cut);
    }

    // Destruktor
    virtual ~TCP_Pkt() {
	chk_mem("TCP_Pkt->del 0x%hX\r\n", get_types());
	if (hdr) {
	  delete hdr;
	}
     }

    virtual void show_p(uchar kind) {
      if (!hdr) {
        return;
      }
      if ( kind == SRC ) {
	show_port(hdr->d_port, T_TCP);
      } else if ( kind == DST ) {
	show_port(hdr->s_port, T_TCP);
      }
    }

    virtual void header(ushort flags, char recursive=TRUE) {
      if ( hdr && (flags & T_TCP)) {
	show_port(hdr->s_port,T_TCP);
	g_puts(" ->");
	show_port(hdr->d_port, T_TCP);
      }
      if (recursive && next) {
        next->header(flags);
      }
    }

    virtual void show(ushort flags);

    virtual ushort get_types() {
      return ((next)?(next->get_types()):0) | T_TCP;
    }
};


#endif /* IP_PKT */
