#ifndef __UDP_PKT
#define __UDP_PKT
#  include  "anl_pkt.h"
#  include  "gen_pkt.h"

class UDP_Pkt : public PortPkt {

protected:
    UDP_hdr * hdr;
    enum {ADDRSIZ=2}; // C. portu
    // Vlastni metody

    void swap_hdr() {
      if (!hdr) {
        return;
      }
      hdr->d_port = SWAP16(hdr->d_port);
      hdr->s_port = SWAP16(hdr->s_port);
      hdr->len    = SWAP16(hdr->len);
      hdr->chksum = SWAP16(hdr->chksum);
    }

    virtual AnlPkt * parse_pkt(const uchar * pdata, uint32 psize, char cut) {
      if ( psize >= sizeof(UDP_hdr) ) {
        hdr = new UDP_hdr();
        memcpy(hdr, pdata, sizeof(UDP_hdr));
        pdata += sizeof(UDP_hdr);
        psize -= sizeof(UDP_hdr);
	swap_hdr();
      }
      return new Pkt_Data(pdata, psize, cut);
    }

public:
    // Konstruktory
    UDP_Pkt(const uchar * pktdata, const uint32 psize, char cut) {
      hdr = NULL;
      next=parse_pkt(pktdata, psize, cut);
    }

    // Destruktor
    virtual ~UDP_Pkt() {
	chk_mem("UDP_Pkt->del 0x%hX\r\n", get_types());
	if (hdr) {
	  delete hdr;
	}
     }

    // Virtualni metody
    virtual void show_p(uchar kind) {
      if (!hdr) {
        return;
      }
      if ( kind == SRC ) {
	show_port(hdr->d_port, T_UDP);
      } else if ( kind == DST ) {
	show_port(hdr->s_port, T_UDP);
      }
    }

    virtual void header(ushort flags, char recursive=TRUE) {
      if ( hdr &&(flags & T_UDP)) {
	show_port(hdr->s_port, T_UDP);
	g_puts(" ->");
	show_port(hdr->d_port, T_UDP);
      }
      if (recursive && next) {
        next->header(flags);
      }
    }

    virtual void show(ushort flags) {
      if ( hdr && (flags & T_UDP)) {
        g_puts("UDP:");
        header(flags, FALSE);
        g_printf(" Delka: %hu  CHK: 0x%04hX", hdr->len, hdr->chksum);
        g_putc('\n');
      }
      if (next) {
        if ( !( flags & T_DATA) && (flags &  S_INTER) && hdr && 
             (inter_p(hdr->s_port, T_UDP, SRC) | 
              inter_p(hdr->d_port, T_UDP, DST)  ) == ALWAYS) {
           flags |= T_DATA;
        }
        next->show(flags);
      }
    }

    virtual ushort get_types() {
      return ((next)?(next->get_types()):0) | T_UDP;
    }
};


#endif /* IP_PKT */
